#include "../misc/common.h"

#include "pmatrix.h"

namespace math
{
  pmatrix::pmatrix() : nn(0), mm(0), v(0x0) {}

  pmatrix::pmatrix(int n, int m) : nn(n), mm(m), v(n>0 ? new posynomial*[n] : NULL)
  {
    int i,nel=m*n;
    if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
    for (i=1;i<n;i++) v[i] = v[i-1] + m;
  }

  pmatrix::pmatrix(int n, int m, posynomial &p) : nn(n), mm(m), 
							v(n>0 ? new posynomial*[n] : NULL)
  {
	int i,j,nel=m*n;
	if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
	for (i=1; i< n; i++) v[i] = v[i-1] + m;
	for (i=0; i< n; i++) for (j=0; j<m; j++) v[i][j] = p;
  }
  
  bool pmatrix::insert(int i, int j, posynomial& p)
  {
    if (i < 0 || i >= nn) {
      return false;
    }
    if (j < 0 || j >= nn) {
      return false;
    }
    v[i][j] = p;
    return true;
  }
  
  pmatrix::pmatrix(const pmatrix &rhs) : nn(rhs.nn), mm(rhs.mm), 
					 v(nn>0 ? new posynomial*[nn] : NULL)
  {
    int i,j,nel=mm*nn;
    if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
    for (i=1; i< nn; i++) v[i] = v[i-1] + mm;
    for (i=0; i< nn; i++) for (j=0; j<mm; j++) v[i][j] = rhs[i][j];
  }
  
  pmatrix & pmatrix::operator=(const pmatrix &rhs)
  // postcondition: normal assignment via copying has been performed;
  //		if matrix and rhs were different sizes, matrix
  //		has been resized to match the size of rhs
  {
    if (this != &rhs) {
      int i,j,nel;
      if (nn != rhs.nn || mm != rhs.mm) {
	if (v != NULL) {
	  delete[] (v[0]);
	  delete[] (v);
	}
	nn=rhs.nn;
	mm=rhs.mm;
	v = nn>0 ? new posynomial*[nn] : NULL;
	nel = mm*nn;
	if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
	for (i=1; i< nn; i++) v[i] = v[i-1] + mm;
      }
      for (i=0; i< nn; i++) for (j=0; j<mm; j++) v[i][j] = rhs[i][j];
    }
    return *this;
  }
  
  void pmatrix::resize(int newn, int newm)
  {
    int i,nel;
    if (newn != nn || newm != mm) {
      if (v != NULL) {
	delete[] (v[0]);
	delete[] (v);
      }
      nn = newn;
      mm = newm;
      v = nn>0 ? new posynomial*[nn] : NULL;
      nel = mm*nn;
      if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
      for (i=1; i< nn; i++) v[i] = v[i-1] + mm;
    }
  }

  void pmatrix::assign(int newn, int newm, const posynomial& a)
  {
    int i,j,nel;
    if (newn != nn || newm != mm) {
      if (v != NULL) {
	delete[] (v[0]);
	delete[] (v);
      }
      nn = newn;
      mm = newm;
      v = nn>0 ? new posynomial*[nn] : NULL;
      nel = mm*nn;
      if (v) v[0] = nel>0 ? new posynomial[nel] : NULL;
      for (i=1; i< nn; i++) v[i] = v[i-1] + mm;
    }
    for (i=0; i< nn; i++) for (j=0; j<mm; j++) v[i][j] = a;
  }

  pmatrix::~pmatrix()
  {
    if (v != NULL) {
      delete[] (v[0]);
      delete[] (v);
    }
  }
  
  void pmatrix::save(FILE *fp, misc::cstr_t var_prefix, misc::cstr_t format ) const
  {
    printf("Total rows , cols : %d, %d\n", nrows(), ncols());
    
    for(int i = 0; i < nrows(); i++){ 
      for(int j = 0; j < ncols(); j++) {
	posynomial p = v[i][j];
	std::string pstr = p.to_string(var_prefix, format);
	fprintf(fp,"%s",pstr.c_str());
	fprintf(fp,"\t");
      }
      fprintf(fp,"\n");
    }
  }

  void pmatrix::save(FILE * fp) const
  {
    fprintf(fp, "%lu %lu", nrows(), ncols());
    
    fprintf(fp,"\n");
    
    for(int i = 0; i < nrows(); i++){
      for(int j = 0; j < ncols(); j++) {
	posynomial p = v[i][j];
	p.save(fp);
      }
    }
  }

  bool pmatrix::load(FILE *fp)
  {
    int n = 0, m = 0;
    if ( fscanf(fp, "%d %d", &n, &m) != 2 )
      return false;
    
    fprintf(stdout, "pmatrix load rows : %d cols : %d\n", n, m);
    
    resize(n, m);
    
    for(int i = 0; i < n; i++){
      for(int j = 0; j < m; j++){
	if ( ! v[i][j].load(fp)  )
	  return false;
	//std::string p_str = v[i][j].to_string("x");
	//fprintf(stdout, "%s\n", p_str.c_str());
      }
    }
    return true;
  }

}; //namespace math
