#include <iostream>
#include <vector>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <algorithm>
#include "nr3.h"
#include "mins.h"

//We are interested only in the real roots
double solve_cubic2(std::vector<double> coeffs, double lb)
{
  double a = coeffs[3];
  double b = coeffs[2];
  double c = coeffs[1];
  double d = coeffs[0];
  
  double p = c/a - b*b/a/a/3;
  double q = (2*b*b*b/a/a/a - 9*b*c/a/a + 27*d/a) / 27;
  
  double pi = 3.141592654;
  
  double D = p*p*p/27 + q*q/4;
  double y1 = 0.0, y2 = 0.0, y3 = 0.0;
  
  double tfinal = b/a/3;
  
  //3 real unequal roots -- use trigonometric formulations
  if ( D < 0 ) {
    double phi = acos(-q/2/sqrt(abs(p*p*p)/27));
    double temp1=2*sqrt(abs(p)/3);
    y1 =  temp1*cos(phi/3);
    y2 = -temp1*cos((phi+pi)/3);
    y3 = -temp1*cos((phi-pi)/3);
    
    y1 -= tfinal;
    y2 -= tfinal;
    y3 -= tfinal;
    printf("Roots y1 %e y2 %e y3 %e\n", y1, y2, y3);
  }
  else if ( D == 0 ){
    double temp = -q/2;
    double u = pow(abs(temp), 1.0/3.0);
    if ( temp < 0 ) u = -u;
    y1 = 2*u;
    y2 = -u;
    y3 = -u;
    
    y1 -= tfinal;
    y2 -= tfinal;
    y3 -= tfinal;
    printf("Roots y1 %e y2 %e y3 %e\n", y1, y2, y3);
  } else {
    double temp1 = -q/2 + sqrt(D);
    double temp2 = -q/2 - sqrt(D);
    
    double u = pow(abs(temp1), 1.0/3.0);
    double v = pow(abs(temp2), 1.0/3.0);
    if ( temp1 < 0 ) u = -u;
    if ( temp2 < 0 ) v = -v;
    y1 = u + v;
    y2 = -1;
    y3 = -1;
    y1 -= tfinal;
    printf("Roots y1 %e y2 %e y3 %e\n", y1, y2, y3);
  }
  
  std::vector<double> positive_roots;
  if ( y1 > 0 )
    positive_roots.push_back(y1);
  if ( y2 > 0 )
    positive_roots.push_back(y2);
  if ( y3 > 0 )
    positive_roots.push_back(y3);
  
  if ( positive_roots.size() > 1 ) {
    //now we check which roots lies between lb and ub
    std::sort(positive_roots.begin(), positive_roots.end());
    printf("Needed to sort the roots \n");
    for(size_t i = 0; i < positive_roots.size(); i++) {
      if ( positive_roots[i] > lb )
	return positive_roots[i];
    }
    return positive_roots[positive_roots.size()-1];
  } else {
    return positive_roots[0];
  }
}

struct Funcd {
  Doub operator() (const Doub x) {
    double ret = x*x*x + 3*x*x + 3*x + 1;
    return ret;
  }
  
  Doub df (const Doub x) {
    double ret = 3*x*x + 6*x + 3;
    return ret;
  }
};

struct func{
    std::vector<double> coeffs;
    
    func(size_t n) {
      coeffs.resize(n, 0.0);
    }
    
    func(std::vector<double> c){
      coeffs = c;
    }
    
    double operator() (const double x)
    {
      double ret = 0.0;
      for(size_t i = 0; i < coeffs.size(); i++)
	ret += coeffs[i]*pow(x,(double)i);
      return ret;
    }
    
    double df(const double x)
    {
      double ret = 0.0, cterm = 0.0, pterm = 0.0;
      for(size_t i = 0; i < coeffs.size(); i++){
	double c = coeffs[i];
	cterm = c*i;
	if ( cterm == 0 ) continue;
	else pterm = pow(x,(double)(i-1));
	ret += cterm*pterm;
      }
      return ret;
    }
    
    void resize(size_t n) {
      coeffs.resize(n);
    }
};
    

template <class T>
Doub rtnewt(T &funcd, const Doub x1, const Doub x2, const Doub xacc) 
{
  const Int JMAX=20;
  Doub rtn=0.5*(x1+x2);
  for (Int j=0;j<JMAX;j++) {
    Doub f=funcd(rtn);
    Doub df=funcd.df(rtn);
    Doub dx=f/df;
    rtn -= dx;
    if ((x1-rtn)*(rtn-x2) < 0.0)
      assert(false && "Jumped out of brackets in rtnewt");
    if (abs(dx) < xacc) return rtn;
  }
  assert(false && "Maximum number of iterations exceeded in rtnewt");
}

template <class T>
Doub rtsafe(T &funcd, const Doub x1, const Doub x2, const Doub xacc) 
{
  const Int MAXIT=100;
  Doub xh,xl;
  Doub fl=funcd(x1);
  Doub fh=funcd(x2);
  if ((fl > 0.0 && fh > 0.0) || (fl < 0.0 && fh < 0.0))
    throw("Root must be bracketed in rtsafe");
  if (fl == 0.0) return x1;
  if (fh == 0.0) return x2;
  if (fl < 0.0) {
    xl=x1;
    xh=x2;
  } else {
    xh=x1;
    xl=x2;
  }
  Doub rts=0.5*(x1+x2);
  Doub dxold=abs(x2-x1);
  Doub dx=dxold;
  Doub f=funcd(rts);
  Doub df=funcd.df(rts);
  for (Int j=0;j<MAXIT;j++) {
    if ((((rts-xh)*df-f)*((rts-xl)*df-f) > 0.0)
	|| (abs(2.0*f) > abs(dxold*df))) {
      dxold=dx;
      dx=0.5*(xh-xl);
      rts=xl+dx;
      if (xl == rts) return rts;
    } else {
      dxold=dx;
      dx=f/df;
      Doub temp=rts;
      rts -= dx;
      if (temp == rts) return rts;
    }
    if (abs(dx) < xacc) return rts;
    Doub f=funcd(rts);
    //Doub df=funcd.df(rts);
    if (f < 0.0)
      xl=rts;
    else
      xh=rts;
  }
  assert(false && "Maximum number of iterations exceeded in rtsafe");
}

double
solve_cubic(std::vector<double> coeffs)
{
  double p = coeffs[2]/coeffs[3];
  double q = coeffs[1]/coeffs[3];
  double r = coeffs[0]/coeffs[3];

  double a = q - (p*p)/3;
  double b = r - (p*q)/3 + (2*p*p*p)/27;
  
  double D = b*b/4 + a*a*a/27;
  
  printf("Determinant of cubic equation : %e\n", D);
  
  assert( D >= 0 && "Determinant of cubic solution must be greater than zero to get one real root");
  
  double A = pow( b/2 + sqrt(D), 0.33);
  double B = pow( b/2 - sqrt(D), 0.33);
  
  double ret = A + B - p/3;
  
  return ret;
}

int main(int argc, char * argv[])
{
  //c[0] : -4.610185e+01, c[1] : 0.000000e+00, 
  //c[2] : 3.501700e+01, c[3] : 1.865368e+00, 
  func f(4);
  f.coeffs[0] = -4.610185e+01;
  f.coeffs[1] = 0.0;
  f.coeffs[2] = 3.501700e+01;
  f.coeffs[3] = 1.865368e+00;
  
  std::vector<double> c;
  c.resize(4);
  c[0] = -9.091592e+01, c[1] = 0.000000e+00, c[2] = 7.052230e+00, c[3] = 1.865368e+00;
#if 1
  double root = solve_cubic2(c, 1.69);
  printf("Root : %g\n", root);
#endif
 
#if 0
  double min = rtsafe(f, 0, 100, 1e-6);
  printf("Minimum : %e\n", min);
#endif

#if 0
  Dbrent dbrent;
  Funcd f;
  dbrent.bracket(0,100,f);
  double xmin = dbrent.minimize(f);
  printf("Minimum : %e, fmin : %e\n",xmin, dbrent.fmin);
#endif
}
